@@ -18,20 +18,21 @@
 import os
 import os.path
 import logging
+import mimetypes
 from flask import (
     abort,
-    flash,
+    jsonify,
     request,
     send_file
 )
-from airflow.models.errors import ImportError
-from airflow_code_editor.commons import ROUTE
-from airflow_code_editor.utils import (
-    get_root_folder,
-    execute_git_command,
-    normalize_path,
+from airflow_code_editor.commons import (
     HTTP_404_NOT_FOUND
 )
+from airflow_code_editor.utils import (
+    git_absolute_path,
+    execute_git_command,
+    normalize_path
+)
 
 __all__ = [
     'AbstractCodeEditorView'
@@ -43,62 +44,33 @@ class AbstractCodeEditorView(object):
     def _index(self, session=None):
         return self._render('index')
 
-    def _load(self, path):
+    def _save(self, session=None, path=None):
         try:
-            code = None
-            cwd = get_root_folder()
-            fullpath = os.path.join(cwd, path)
-            # Read code
-            with open(fullpath, 'r') as f:
-                code = f.read().rstrip('\n')
-        except Exception as ex:
-            logging.error(ex)
-            flash('Error loading file [{path}]'.format(path=path),
-                  'error')
-        finally:
-            return code
-
-    def _save(self, path):
-        try:
-            code = None
-            code = request.form['code']
+            data = request.form['data']
             # Newline fix (remove cr)
-            code = code.replace('\r', '').rstrip()
-            cwd = get_root_folder()
-            fullpath = os.path.join(cwd, path)
+            data = data.replace('\r', '').rstrip()
+            fullpath = git_absolute_path(path)
+            os.makedirs(os.path.dirname(fullpath), exist_ok=True)
             with open(fullpath, 'w') as f:
-                f.write(code)
+                f.write(data)
                 f.write('\n')
-            flash('File [{path}] saved successfully'.format(path=path),
-                  'success')
+            return jsonify({
+                'path': normalize_path(path)
+            })
         except Exception as ex:
             logging.error(ex)
-            flash('Error saving file [{path}]'.format(path=path),
-                  'error')
-        finally:
-            return code
-
-    def _editor(self, session=None, path=None):
-        path = normalize_path(path)
-        try:
-            code = None
-            # Display import error
-            for ie in session.query(ImportError).all():
-                if ie.filename == path:
-                    flash('Broken DAG: [{ie.filename}] {ie.stacktrace}'.format(ie=ie),
-                          'dag_import_error')
-            # Load or Save DAG
-            if 'code' in request.form:
-                code = self._save(path)
+            if hasattr(ex, 'strerror'):
+                message = ex.strerror
+            elif hasattr(ex, 'message'):
+                message = ex.message
             else:
-                code = self._load(path)
-        finally:
-            return self._render(
-                'editor',
-                code=code,
-                path=path,
-                back=ROUTE,
-                root=request.args.get('root'))
+                message = str(ex)
+            return jsonify({
+                'path': normalize_path(path),
+                'error': {
+                    'message': 'Error saving {path}: {message}'.format(path=path, message=message)
+                }
+            })
 
     def _git_repo(self, session, path):
         if request.method == 'POST':
@@ -115,14 +87,26 @@ class AbstractCodeEditorView(object):
         git_args = request.form.getlist('args[]')
         return execute_git_command(git_args)
 
-    def _download(self, session, path):
+    def _load(self, session, path):
         " Send the contents of a file to the client "
         try:
-            cwd = get_root_folder()
-            fullpath = os.path.abspath(os.path.join(cwd, path))
-            if not fullpath.startswith(cwd):
-                raise Exception('Not in root path')
-            return send_file(fullpath, as_attachment=True)
+            path = normalize_path(path)
+            if path.startswith('~git/'):
+                # Download git blob - path = '~git/<hash>/<name>'
+                _, path, filename = path.split('/', 3)
+                response = execute_git_command(["cat-file", "-p", path])
+                response.headers['Content-Disposition'] = 'attachment; filename="%s"' % filename
+                try:
+                    content_type = mimetypes.guess_type(filename)[0]
+                    if content_type:
+                        response.headers['Content-Type'] = content_type
+                except Exception:
+                    pass
+                return response
+            else:
+                # Download file
+                fullpath = git_absolute_path(path)
+                return send_file(fullpath, as_attachment=True)
         except Exception as ex:
             logging.error(ex)
             abort(HTTP_404_NOT_FOUND)
