@@ -20,6 +20,8 @@ import logging
 import subprocess
 import threading
 import shlex
+from datetime import datetime
+from collections import namedtuple
 from flask import make_response
 from flask_login import current_user
 from airflow import configuration
@@ -28,7 +30,8 @@ from airflow_code_editor.commons import (
     HTTP_404_NOT_FOUND,
     SUPPORTED_GIT_COMMANDS,
     PLUGIN_NAME,
-    PLUGIN_DEFAULT_CONFIG
+    PLUGIN_DEFAULT_CONFIG,
+    ROOT_MOUNTPOUNT
 )
 
 __all__ = [
@@ -36,6 +39,7 @@ __all__ = [
     'get_plugin_config',
     'get_plugin_boolean_config',
     'get_root_folder',
+    'git_absolute_path',
     'execute_git_command'
 ]
 
@@ -55,12 +59,12 @@ def normalize_path(path):
 
 def get_plugin_config(key):
     " Get a plugin configuration/default for a given key "
-    return configuration.get(PLUGIN_NAME, key, fallback=PLUGIN_DEFAULT_CONFIG[key])
+    return configuration.conf.get(PLUGIN_NAME, key, fallback=PLUGIN_DEFAULT_CONFIG[key])
 
 
 def get_plugin_boolean_config(key):
     " Get a plugin boolean configuration/default for a given key "
-    return configuration.getboolean(PLUGIN_NAME, key, fallback=PLUGIN_DEFAULT_CONFIG[key])
+    return configuration.conf.getboolean(PLUGIN_NAME, key, fallback=PLUGIN_DEFAULT_CONFIG[key])
 
 
 def prepare_response(git_cmd, result=None, stderr=None, returncode=0):
@@ -104,7 +108,20 @@ def prepare_git_env():
 
 def get_root_folder():
     " Return the configured root folder or Airflow DAGs folder "
-    return os.path.abspath(get_plugin_config('root_directory') or configuration.get('core', 'dags_folder'))
+    return os.path.abspath(get_plugin_config('root_directory') or configuration.conf.get('core', 'dags_folder'))
+
+
+def git_absolute_path(git_path):
+    " Git relative path to absolute path "
+    path = normalize_path(git_path)
+    if path.startswith('~'):
+        # Expand paths beginning with '~'
+        prefix, remain = path.split('/', 1) if '/' in path else (path, '')
+        try:
+            return os.path.join(mount_points[prefix[1:]].path, remain)
+        except KeyError:
+            pass
+    return os.path.join(get_root_folder(), path)
 
 
 _execute_git_command_lock = threading.Lock()
@@ -122,6 +139,10 @@ def execute_git_command(git_args):
                 stdout = git_ls_local(git_args)
                 stderr = None
                 returncode = 0
+            elif git_cmd == 'mounts':
+                stdout = git_mounts(git_args)
+                stderr = None
+                returncode = 0
             elif git_cmd in SUPPORTED_GIT_COMMANDS:
                 git_default_args = shlex.split(get_plugin_config('git_default_args'))
                 cmd = [get_plugin_config('git_cmd')] + git_default_args + git_args
@@ -153,13 +174,13 @@ def execute_git_command(git_args):
 
 def git_ls_local(git_args):
     " 'git ls-tree' like output for local folders "
-    cwd = get_root_folder()
     long_ = False
     if '-l' in git_args or '--long' in git_args:
         git_args = [arg for arg in git_args if arg not in ('-l', '--long')]
         long_ = True
     path = git_args[1] if len(git_args) > 1 else ''
-    dirpath = os.path.join(cwd, normalize_path(path))
+    path = normalize_path(path.split('#', 1)[0])
+    dirpath = git_absolute_path(path)
     result = []
     for name in sorted(os.listdir(dirpath)):
         if name.startswith('.') or name == '__pycache__':
@@ -172,14 +193,20 @@ def git_ls_local(git_args):
         else:
             type_ = 'blob'
             size_ = s.st_size
-        relname = fullname[len(cwd):]
+        relname = os.path.join('/', path, name)  # fullname[len(cwd):]
         if long_:
-            result.append('%06o %s %s %8s\t%s' % (s.st_mode, type_, relname, size_, name))
+            mtime = datetime.utcfromtimestamp(s.st_mtime).isoformat()[:16]
+            result.append('%06o %s %s#%s %8s\t%s' % (s.st_mode, type_, relname, mtime, size_, name))
         else:
             result.append('%06o %s %s\t%s' % (s.st_mode, type_, relname, name))
     return '\n'.join(result)
 
 
+def git_mounts(git_args):
+    " List mountpoints "
+    return '\n'.join(sorted(k for k, v in mount_points.items() if not v.default))
+
+
 def init_git_repo():
     " Initialize the git repository in root folder "
     cwd = get_root_folder()
@@ -192,26 +219,29 @@ def init_git_repo():
     subprocess.call(['git', 'commit', '-m', 'Initial commit'], cwd=cwd, env=prepare_git_env())
 
 
-assert(normalize_path('/') == '')
-assert(normalize_path('/../') == '')
-assert(normalize_path('../') == '')
-assert(normalize_path('../../') == '')
-assert(normalize_path('../..') == '')
-assert(normalize_path('/..') == '')
+MountPoint = namedtuple('MountPoint', 'path default')
 
-assert(normalize_path('//') == '')
-assert(normalize_path('////../') == '')
-assert(normalize_path('..///') == '')
-assert(normalize_path('..///../') == '')
-assert(normalize_path('..///..') == '')
-assert(normalize_path('//..') == '')
 
-assert(normalize_path('/aaa') == 'aaa')
-assert(normalize_path('/../aaa') == 'aaa')
-assert(normalize_path('../aaa') == 'aaa')
-assert(normalize_path('../../aaa') == 'aaa')
-assert(normalize_path('../../aaa') == 'aaa')
-assert(normalize_path('/../aaa') == 'aaa')
+def read_mount_points_config():
+    " Return the plugin configuration "
+    global mount_points
+    config = {
+        ROOT_MOUNTPOUNT: MountPoint(path=get_root_folder(), default=True)
+    }
+    i = 0
+    # Iterate over the configurations
+    while True:
+        suffix = str(i) if i != 0 else ''  # the first configuration doesn't have a suffix
+        try:
+            if not configuration.conf.has_option(PLUGIN_NAME, 'mount{}_name'.format(suffix)):
+                break
+        except Exception:  # backports.configparser.NoSectionError and friends
+            break
+        name = configuration.conf.get(PLUGIN_NAME, 'mount{}_name'.format(suffix))
+        path = configuration.conf.get(PLUGIN_NAME, 'mount{}_path'.format(suffix))
+        config[name] = MountPoint(path=path, default=False)
+        i = i + 1
+    mount_points = config
 
-assert(normalize_path('/aaa') == 'aaa')
-assert(normalize_path('aaa') == 'aaa')
+
+read_mount_points_config()
